import string
import gtk
import gobject
import rhpl.monitor

TRUE = 1
FALSE = 0

class MonitorDialog:
    def selection_changed(self, selection):
        if self.changing_selection:
            return
        (store, iter) = selection.get_selected()
        if iter == None:
            return

        if store.get_value(iter, 1) != None:
            model = store.get_value(iter, 0)
            vsync = store.get_value(iter, 2)
            hsync = store.get_value(iter, 3)
            self.model_entry.set_text(model)
            self.hsync_entry.set_text(hsync)
            self.vsync_entry.set_text(vsync)

    def probe_monitor(self, widget):
        if self.probed_path:
            self.goto_path(self.probed_path)
        model = self.monitor.getMonitorName()
        hsync = self.monitor.getMonitorHorizSync(TRUE)
        vsync = self.monitor.getMonitorVertSync(TRUE)
        self.model_entry.set_text(model)
        self.hsync_entry.set_text(hsync)
        self.vsync_entry.set_text(vsync)
            
    def __init__(self, xml, monitor, probed_id):
        self.xml = xml
        self.monitor = monitor

        self.model_entry = self.xml.get_widget("monitor_name_entry")
        self.hsync_entry = self.xml.get_widget("monitor_hsync_entry")
        self.vsync_entry = self.xml.get_widget("monitor_vsync_entry")

        self.probed_path = None

        self.changing_selection = 0
        
        dialog = self.xml.get_widget("monitor_dialog")
        dialog.set_transient_for(self.xml.get_widget("display_dialog"))
        dialog.set_modal(TRUE)

        monitor_store = gtk.TreeStore(gobject.TYPE_STRING, gobject.TYPE_STRING,
                                      gobject.TYPE_STRING, gobject.TYPE_STRING)
        self.monitor_store = monitor_store

        self.models = {}
        db = monitor.readMonitorsDB()
        l = db.keys()
        l.sort()
        for maker in ["Generic LCD Display", "Generic CRT Display", "Generic"] + l:
	    if maker not in l:
		continue
	    
            parent = monitor_store.append(None)
            monitor_store.set_value(parent, 0, maker)
            for mon in db[maker]:
                iter = monitor_store.append(parent)
                monitor_store.set_value(iter, 0, mon[0])
                monitor_store.set_value(iter, 1, mon[1])
                monitor_store.set_value(iter, 2, mon[2])
                monitor_store.set_value(iter, 3, mon[3])
                
                if probed_id and string.upper(mon[1]) == string.upper(probed_id):
                    self.probed_path = monitor_store.get_path(iter) 

                self.models[string.upper(mon[0])] = monitor_store.get_path(iter) 

        tree_view = self.xml.get_widget("monitor_tree")
        tree_view.set_model(monitor_store)
        
        selection = tree_view.get_selection()
        selection.connect("changed", self.selection_changed)

        self.col = gtk.TreeViewColumn(None, gtk.CellRendererText(), text=0)
        tree_view.append_column(self.col)

        button = self.xml.get_widget("monitor_probe_button")
        button.connect("clicked", self.probe_monitor)
        button.set_sensitive(probed_id != "Unprobed Monitor")

    def goto_path(self, path):
        tree_view = self.xml.get_widget("monitor_tree")
        tree_view.expand_row(path[:-1], TRUE)
        self.changing_selection = 1
        selection = tree_view.get_selection()
        selection.unselect_all()
        iter = self.monitor_store.get_iter(path)
        selection.select_iter(iter)
        self.changing_selection = 0
        tree_view.set_cursor(path, self.col, FALSE)
        tree_view.scroll_to_cell(path, self.col, TRUE, 0.5, 0.0)
        tree_view.grab_focus ()

    def hydrate(self, state):
        self.model_entry.set_text(state.get_monitor_name())
        self.hsync_entry.set_text(state.get_hsync_or_probed())
        self.vsync_entry.set_text(state.get_vsync_or_probed())

    def dehydrate(self, state):
        model = self.model_entry.get_text()
        state.set_monitor_name(model)
        hsync = self.hsync_entry.get_text()
        state.set_hsync(hsync)
        vsync = self.vsync_entry.get_text()
        state.set_vsync(vsync)

    def run(self):
        model = self.model_entry.get_text()
        key = string.upper(model)
        if self.models.has_key(key):
            self.goto_path(self.models[key])
        
        dialog = self.xml.get_widget("monitor_dialog")
        dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
        dialog.present()
        while 1:
            res = dialog.run()
            if res != gtk.RESPONSE_OK:
                dialog.hide()
                return FALSE
            
            hsync = self.hsync_entry.get_text()
            if not rhpl.monitor.isValidSyncRange(hsync):
                m = gtk.MessageDialog(dialog, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Invalid hsync specification"))
                m.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
                m.run()
                m.destroy()
                continue
            
            vsync = self.vsync_entry.get_text()
            if not rhpl.monitor.isValidSyncRange(vsync):
                m = gtk.MessageDialog(dialog, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Invalid vsync specification"))
                m.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
                m.run()
                m.destroy()
                continue

            break
        
        dialog.hide()
        return TRUE

