import gtk
import gobject
from rhpl.translate import _, N_,textdomain
#from rhpl.xhwstate import *
import string

TRUE = 1
FALSE = 0

class VideocardDialog:
    #TODO: Is this list correct?
    drivers = ["mga", "ati", "chips", "cirrus", "cyrix", "fbdev",
               "glint", "i128", "i740", "i810", "mga", "neomagic",
               "newport", "nv", "r128", "radeon", "rendition", "s3",
               "s3virge", "savage", "siliconmotion", "sis", "tdfx",
               "trident", "tseng", "vesa", "vga", "vmware"]

    def __init__(self, xml, videocard, hardware_state):
        self.xml = xml
        self.videocard = videocard
        self.hardware_state = hardware_state
 
        self.changing_selection = 0
        
        dialog = xml.get_widget("videocard_dialog")
        dialog.set_transient_for(xml.get_widget("display_dialog"))
        dialog.set_modal(TRUE)

        optionmenu = xml.get_widget("videocard_mem_option_menu")
        menu = gtk.Menu()
        for ram in videocard.possible_ram_sizes():
            if ram % 1024 != 0:
                m = (ram, repr(ram) +" Kb")
            else:
                m = (ram, repr(ram/1024) +" Mb")
            item = gtk.MenuItem(m[1])
            item.set_data("mem-size", m[0])
            item.show()
            menu.append(item)
        optionmenu.set_menu(menu)

        xml.get_widget("videocard_driver_combo"). set_popdown_strings(self.drivers)

        videocard_store = gtk.ListStore(gobject.TYPE_STRING)
        self.videocard_store = videocard_store

        iter = videocard_store.append()
        videocard_store.set_value(iter, 0, "Custom")
        card_list = videocard.cardsDB().keys()
        card_list.sort()
        for name in card_list:
            iter = videocard_store.append()
            videocard_store.set_value(iter, 0, name)

        tree_view = xml.get_widget("videocard_tree")
        tree_view.set_model(videocard_store)
        selection = tree_view.get_selection()
        # Select the first before setting up callback, to avoid first callback.
        selection.select_iter(videocard_store.get_iter_first())
        selection.connect("changed", self.selection_changed)

        self.vc_col = gtk.TreeViewColumn(None, gtk.CellRendererText(), text=0)
        tree_view.append_column(self.vc_col)

        options_store = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
        self.options_store = options_store
        tree_view = xml.get_widget("videocard_options_tree")
        tree_view.set_model(options_store)
        cell_renderer = gtk.CellRendererText()
        cell_renderer.set_property("editable", TRUE)
        cell_renderer.connect("edited", self.option_name_edited)
        self.col = gtk.TreeViewColumn(_("option"), cell_renderer, text=0)
        tree_view.append_column(self.col)
        cell_renderer = gtk.CellRendererText()
        cell_renderer.set_property("editable", TRUE)
        cell_renderer.connect("edited", self.option_value_edited)
        self.col = gtk.TreeViewColumn(_("value"), cell_renderer, text=1)
        tree_view.append_column(self.col)
        self.options_tree_view = tree_view

        self.type_entry = xml.get_widget("videocard_type_entry")
        self.driver_entry = xml.get_widget("videocard_driver_entry")
        self.mem_check = xml.get_widget("videocard_mem_check")
        self.mem_optionmenu = xml.get_widget("videocard_mem_option_menu")

        self.mem_check.connect("toggled", self.custom_mem_toggled)

        button = xml.get_widget("videocard_probe_button")
        button.connect("clicked", self.probe_videocard)
        primary = videocard.primaryCard()
        button.set_sensitive(primary != None and
                             primary.getCardData() != None)

        button = xml.get_widget("videocard_add_option_button")
        button.connect("clicked", self.add_option)
        button = xml.get_widget("videocard_remove_option_button")
        button.connect("clicked", self.remove_option)

    def parse_lines(self, lines):
        for line in string.split(lines, "\n"):
            items = self.hardware_state.tokenize_line(line)
            if len(items) > 1 and string.upper(items[0]) == "OPTION":
                iter = self.options_store.append()
                self.options_store.set_value(iter, 0, items[1])
                if len(items) > 2:
                    self.options_store.set_value(iter, 1, items[2])
                else:
                    self.options_store.set_value(iter, 1, "")
            if len(items) == 2 and string.upper(items[0]) == "VIDEORAM":
                self.set_custom_memory(TRUE)
                self.set_memory_size(string.atoi(items[1]))

    def goto_row(self, cardname):
        iter = self.videocard_store.get_iter_first()
        while iter:
            if self.videocard_store.get_value(iter, 0) == cardname:
                tree_view = self.xml.get_widget("videocard_tree")
                self.changing_selection = 1
                selection = tree_view.get_selection()
                selection.select_iter(iter)
                self.changing_selection = 0
                path = self.videocard_store.get_path(iter)
                tree_view.set_cursor(path, self.vc_col, FALSE)
                tree_view.scroll_to_cell(path, self.vc_col, TRUE, 0.5, 0.0)
                tree_view.grab_focus ()
                break
            iter = self.videocard_store.iter_next(iter)
        

    def set_card_data(self, carddata, goto_row):
        name = carddata["NAME"];
        self.type_entry.set_text(name)
        self.driver_entry.set_text(carddata["DRIVER"])
        self.set_custom_memory(FALSE)
        if goto_row:
            self.goto_row(name)
        self.options_store.clear()
        if carddata.has_key("LINE"):
            lines = carddata["LINE"]
            self.parse_lines(lines)

    def probe_videocard(self, widget):
        primary = self.videocard.primaryCard()
        carddata = primary.getCardData()
        self.set_card_data(carddata, 1)

    def custom_mem_toggled(self, checkbutton):
        self.mem_optionmenu.set_sensitive(checkbutton.get_active())

    def set_custom_memory(self, custom):
        if custom:
            self.mem_check.set_active(TRUE)
            self.mem_optionmenu.set_sensitive(TRUE)
        else:
            self.mem_check.set_active(FALSE)
            self.mem_optionmenu.set_sensitive(FALSE)

    def set_memory_size(self, size):
        if size == 0:
            return
        
        ram_sizes = self.videocard.possible_ram_sizes()
        try:
            index = ram_sizes.index(size)
            self.mem_optionmenu.set_history(index)
        except:
            print _("Warning: Unknown videoram size %d!")%(size)

    def get_memory_size(self):
        if self.mem_check.get_active():
            ram_sizes = self.videocard.possible_ram_sizes()
            index = self.mem_optionmenu.get_history()
            return ram_sizes[index]
        return 0

    def selection_changed(self, selection):
        if self.changing_selection:
            return
        (store, iter) = selection.get_selected()
        if iter == None:
            return
        name = self.videocard_store.get_value(iter, 0)

        if self.videocard.cardsDB().has_key(name):
            carddata = self.videocard.cardsDB()[name]
            self.set_card_data(carddata, 0)
        
    def add_option(self, widget):
        iter = self.options_store.append()
        self.options_store.set_value(iter, 0, _("<Option Name>"))
        self.options_store.set_value(iter, 1, _("<Option Value, click to change>"))
        
    def remove_option(self, widget):
        selection = self.options_tree_view.get_selection()
        (store, iter) = selection.get_selected()
        if iter == None:
            return
        store.remove(iter)

    def option_name_edited(self, cell_renderer, path_string, text):
        iter = self.options_store.get_iter(string.atoi(path_string))
        self.options_store.set_value(iter, 0, text)
        
    def option_value_edited(self, cell_renderer, path_string, text):
        iter = self.options_store.get_iter(string.atoi(path_string))
        self.options_store.set_value(iter, 1, text)
        
        
    def hydrate(self, state):
        name = state.get_videocard_name()
        self.type_entry.set_text(name)
        self.goto_row(name)
        self.driver_entry.set_text(state.get_videocard_driver())
        self.set_memory_size(state.get_videocard_ram_or_probed())
        self.set_custom_memory(state.get_videocard_ram() != 0)
        self.options_store.clear()
        options = state.get_videocard_options()
        for option in options:
            iter = self.options_store.append()
            self.options_store.set_value(iter, 0, option[0])
            if option[1] == None:
                self.options_store.set_value(iter, 1, "")
            else:
                self.options_store.set_value(iter, 1, option[1])

    def dehydrate(self, state):
        state.set_videocard_name(self.type_entry.get_text())
        state.set_videocard_driver(self.driver_entry.get_text())
        state.set_videocard_ram(self.get_memory_size())

        #This doesnt work in the current pygtk if the list is empty. reported upstream
        #iter = self.options_store.get_iter_root()
        list = []
        iter = self.options_store.get_iter_first()
        while iter:
            list.append( (self.options_store.get_value(iter, 0), self.options_store.get_value(iter, 1)) )
            iter = self.options_store.iter_next(iter)
        state.set_videocard_options(list)
        
        
    def run(self):
        dialog = self.xml.get_widget("videocard_dialog")
        dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
        dialog.present()
        while 1:
            res = dialog.run()
            
            if res != gtk.RESPONSE_OK:
                dialog.hide()
                return FALSE
            
            break
        
        dialog.hide()
        return TRUE

