
import os
import xf86config
import gtk
import gtk.glade
import monitorDialog
import screenSizePreview
import videocardDialog
import dpiDialog
from rhpl.translate import _, N_,textdomain
from rhpl.xhwstate import *

TRUE = 1
FALSE = 0

    
class XConfigDialog:
    def setup_sizegroups(self):
        xml = self.xml
        size_group = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
        size_group.add_widget(xml.get_widget("monitor_button"))
        size_group.add_widget(xml.get_widget("dpi_button"))
        size_group.add_widget(xml.get_widget("videocard_button"))
        
        size_group = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
        size_group.add_widget(xml.get_widget("adv_monitor_table_label"))
        size_group.add_widget(xml.get_widget("adv_horiz_table_label"))
        size_group.add_widget(xml.get_widget("adv_dpi_table_label"))
        size_group.add_widget(xml.get_widget("adv_vert_table_label"))
        size_group.add_widget(xml.get_widget("adv_card_table_label"))
        size_group.add_widget(xml.get_widget("adv_card_mem_table_label"))
        size_group.add_widget(xml.get_widget("adv_card_driver_table_label"))

    def calc_dpi_string(self):
        (pw, ph) = self.state.get_physical_size_or_probed()
        res = self.state.get_resolution()
        return dpi_string(res, pw, ph)

    def update_ui(self, *args):
        res = self.res_option_menu.get_menu().get_active().get_data("RESOLUTION")
        (w,h) = resolution_from_string(res)
        self.size_preview.set_resolution(w,h)

        label = self.xml.get_widget("display_label")
        label.set_text(_("%s with %s")%(self.state.get_monitor_name(), self.state.get_videocard_name()))

        label = self.xml.get_widget("monitor_type_label")
        label.set_text(self.state.get_monitor_name())
        
        label = self.xml.get_widget("videocard_type_label")
        label.set_text(self.state.get_videocard_name())

        label = self.xml.get_widget("videocard_mem_label")
        mem = self.state.get_videocard_ram_or_probed()
        if mem == 0:
            label.set_text(_("Unknown"))
        elif mem %1024 != 0:
            label.set_text(_("%d kilobytes")%(mem))
        else:
            label.set_text(_("%d megabytes")%(mem/1024))

        label = self.xml.get_widget("videocard_driver_label")
        label.set_text(self.state.get_videocard_driver())

        checkbutton = self.xml.get_widget("videocard_dri_check")
        dri_supported = self.state.supports_dri()
        checkbutton.set_sensitive(dri_supported)
        checkbutton.set_active(dri_supported and self.state.get_dri_enabled())

        label = self.xml.get_widget("monitor_horiz_label")
        label.set_text(self.state.get_hsync_or_probed())
        
        label = self.xml.get_widget("monitor_vert_label")
        label.set_text(self.state.get_vsync_or_probed())
            
        label = self.xml.get_widget("monitor_dpi_label")
        label.set_text(self.calc_dpi_string())


    def resolution_toggled(self, radiobutton):
        if radiobutton.get_active():
            res = radiobutton.get_data("resolution")
            self.state.set_resolution(res)
            self.update_ui()

    def colordepth_changed(self, menu_item):
        depth = menu_item.get_menu().get_active().get_data("DEPTH")
        self.state.set_colordepth(depth)
        self.update_ui()
        
        if radiobutton.get_active():
            depth = radiobutton.get_data("colordepth")
            self.state.set_colordepth(depth)
            self.update_ui()
            
    def colordepth_toggled(self, radiobutton):
        if radiobutton.get_active():
            depth = radiobutton.get_data("colordepth")
            self.state.set_colordepth(depth)
            self.update_ui()
            
    def dri_toggled(self, checkbutton):
        self.state.set_dri_enabled(checkbutton.get_active())
        self.update_ui()
        
    def setup_ui_resolution(self, current_res):
        box = self.xml.get_widget("resolution_radiobutton_box")
        last = None
        all_resolutions = self.state.get_all_resolutions()

        res_menu = gtk.Menu()
        count = 0
        for res in all_resolutions:
            item = gtk.MenuItem(res)
            item.set_data("RESOLUTION", res)
            res_menu.append(item)
            if res == current_res:
                #Let's remember which position is the current resolution
                current = count
            count = count + 1

        self.res_option_menu.set_menu(res_menu)
        #Select the correct menu item
        self.res_option_menu.set_history(current)
        self.res_option_menu.show_all()
        
    def run(self):
        dialog = self.xml.get_widget("display_dialog")
        dialog.set_position(gtk.WIN_POS_CENTER)
        dialog.show()
        self.dialog_result = 0
        gtk.main()
        dialog.hide()
        return self.dialog_result

    def ok(self, widget):
        res = self.res_option_menu.get_menu().get_active().get_data("RESOLUTION")
        depth = self.depth_option_menu.get_menu().get_active().get_data("DEPTH")

        self.state.set_colordepth(depth)
        self.state.set_resolution(res)

        self.dialog_result = 1
        gtk.main_quit()

    def cancel(self, widget):
        gtk.main_quit()
        
    def delete(self, widget, event):
        gtk.main_quit()
        return 1

    def configure_monitor(self, widget):
        if not self.monitor_dialog:
            self.monitor_dialog = monitorDialog.MonitorDialog(self.xml, self.state.monitor,
                                                self.state.monitor.getMonitorID(TRUE))
            
        self.monitor_dialog.hydrate(self.state)
        if self.monitor_dialog.run():
            self.monitor_dialog.dehydrate(self.state)
            self.state.recalc_mode()
            self.update_ui()
        
    def configure_videocard(self, widget):
        if not self.videocard_dialog:
            self.videocard_dialog = videocardDialog.VideocardDialog(self.xml, self.state.videocard, self.state)
            
        self.videocard_dialog.hydrate(self.state)
        if self.videocard_dialog.run():
            self.videocard_dialog.dehydrate(self.state)
            self.state.recalc_mode()
            self.update_ui()
        
    def configure_dpi(self, widget):
        if not self.dpi_dialog:
            self.dpi_dialog = dpiDialog.DPIDialog(self.xml, self.state.monitor)
            
        self.dpi_dialog.hydrate(self.state)
        if self.dpi_dialog.run():
            self.dpi_dialog.dehydrate(self.state)
            self.update_ui()
        
    def __init__(self, hardware_state):
        # setup UI
        gtk.glade.bindtextdomain("redhat-config-xfree86")
        if os.access("display.glade", os.F_OK):
            xml = gtk.glade.XML("display.glade", domain="redhat-config-xfree86")
        else:
            xml = gtk.glade.XML("/usr/share/redhat-config-xfree86/display.glade", domain="redhat-config-xfree86")
            
        self.state = hardware_state
        self.xml = xml
        self.depth_option_menu = self.xml.get_widget("depth_option_menu")
        self.res_option_menu = self.xml.get_widget("res_option_menu")
        self.res_option_menu.connect("changed", self.update_ui)
        self.low_menu_item = self.xml.get_widget("256_menu_item")
        self.low_menu_item.set_data("DEPTH", 8)
        self.thousands_menu_item = self.xml.get_widget("thousands_menu_item")
        self.thousands_menu_item.set_data("DEPTH", 16)
        self.millions_menu_item = self.xml.get_widget("millions_menu_item")
        self.millions_menu_item.set_data("DEPTH", 24)
        
        self.size_preview = screenSizePreview.ScreenSizePreview(xml.get_widget("screenshot_darea"),200,150)

        ##XXX-remove me later
        (w,h) = resolution_from_string(self.state.get_resolution())
        self.size_preview.set_resolution(w,h)

        self.resolution_radio = {}
        self.videocard_dialog = None
        self.monitor_dialog = None
        self.dpi_dialog = None

        self.setup_sizegroups()
        self.setup_ui_resolution(self.state.get_resolution())

        depth = self.state.get_colordepth()

        available = self.state.available_color_depths()

        if depth in available:
            self.depth_option_menu.set_history(available.index(depth))

        p = gtk.gdk.pixbuf_new_from_file("/usr/share/redhat-config-xfree86/pixmaps/monitor.png")
        xml.get_widget("monitor_icon").set_from_pixbuf(p)

        p = gtk.gdk.pixbuf_new_from_file("/usr/share/redhat-config-xfree86/pixmaps/videocard.png")
        xml.get_widget("video_card_icon").set_from_pixbuf(p)
        
        xml.get_widget("videocard_dri_check").connect("toggled", self.dri_toggled)
        xml.get_widget("monitor_button").connect("clicked", self.configure_monitor)
        xml.get_widget("videocard_button").connect("clicked", self.configure_videocard)
        xml.get_widget("dpi_button").connect("clicked", self.configure_dpi)

        xml.get_widget("ok_button").connect("clicked", self.ok)
        xml.get_widget("cancel_button").connect("clicked", self.cancel)
        xml.get_widget("display_dialog").connect("delete_event", self.delete)
