#!/usr/bin/python2.2

## redhat-config-xfree - an XFree86 configuration tool
## Copyright (c) 2002 Red Hat, Inc.

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


import string
import gobject
import sys
import time
import os
import signal
import xf86config
import math
import getopt
import tempfile
import locale
import rhpl.monitor
import rhpl.videocard
import rhpl.mouse
import rhpl.keyboard
from rhpl.translate import _, N_,textdomain
from rhpl.xhwstate import *

TRUE = 1
FALSE = 0

logFile = "/var/log/XFree86.setup.log"
xserverpid = 0

nameTag = _("Display")
commentTag = _("Configure resolution and colors")

def verbose(str):
    if verbose_output:
        print str

def getArch():
    arch = os.uname()[4]

    if (len (arch) == 4 and arch[0] == 'i' and arch[2:4] == "86"):
        arch = "i386"

    if arch == "sparc64":
        arch = "sparc"

    if arch == "ppc64":
        arch = "ppc"

    if arch == "s390x":
        arch = "s390"

    return arch
                                                      
# TODO:
# probe hsync/vsync seems to give different results than db, which one to trust?
# Better way to detect when DRI is available.

def generate_xconfig(hardware_state):
    xconfig = xf86config.createTemplate()

    if getArch() == "ppc":
        # XXX - hack - here we want to enforce frame buffer requirement on ppc
        #       pick 640x480x8bpp cause that is what frame buffer inits to
        #       when the machine reboots
        hardware_state.set_videocard_name("Framebuffer driver (generic)")
        hardware_state.set_videocard_driver("fbdev")

    hardware_state.merge_into(xconfig)

    keyboard = rhpl.keyboard.Keyboard()
    keyboard.read()

    def get_option(device, optionname):
        res = filter (lambda o: o.name == optionname, device.options)
        if len (res) > 0:
            return res[0]
        else:
            option = xf86config.XF86Option(optionname)
            device.options.insert(option)
            return option

    corekb = xf86config.getCoreKeyboard(xconfig)

    try:
        #Let's assume we know what kind of keyboard this is
        get_option(corekb, "XkbRules").val = keyboard["rules"]
        get_option(corekb, "XkbModel").val = keyboard["model"]
        get_option(corekb, "XkbLayout").val = keyboard["layout"]
    except:
        #Ok, the keyboard from /etc/sysconfig/keyboard isn't in rhpl's list, so let's assume US keyboard
        #Maybe this isn't the best thing to do, but it will work for most people
        keyboard.set("us")
        get_option(corekb, "XkbRules").val = keyboard["rules"]
        get_option(corekb, "XkbModel").val = keyboard["model"]
        get_option(corekb, "XkbLayout").val = keyboard["layout"]

    if keyboard["options"] != "":
        get_option(corekb, "XkbOptions").val = keyboard["options"]
    if keyboard["variant"] != "":
        get_option(corekb, "XkbVariant").val = keyboard["variant"]
       
    corepointer = xf86config.getCorePointer(xconfig)

    #Try probing the mouse
    mouse = rhpl.mouse.Mouse(skipProbe = 0)

    if mouse.getDevice() == None:
        #We couldn't probe the mouse.  Let's try reading /etc/sysconfig/mouse
        mouse = rhpl.mouse.Mouse(skipProbe = 1)

    mouse.read()
    get_option(corepointer, "Device").val = "/dev/" + mouse.getDevice()

    type = mouse.info["XMOUSETYPE"]
    if type == "none":
        type = "PS/2" # No idea, Guess PS/2
    get_option(corepointer, "Protocol").val = type
        
    (typename, emu3) = mouse.get()
    val = "no"
    if emu3:
        val = "yes"
    get_option(corepointer, "Emulate3Buttons").val = val

    if mouse.getDevice() == "input/mice":
        inputs = xconfig.layout[0].inputs
        for i in range(len(inputs)):
            if inputs[i].inputdev == "DevInputMice":
                inputs.remove(i)
                break
    
    return xconfig

def finished_dialog(config_file, backup_file, moved_old_file):
    message = _("Display settings changed\n\n")
    if not xserverpid:
        message = message + _("You need to log out and restart the X server for the changes to take effect.\n")
    if backup_file:
        message = message + _("Configuration was written to %s, original configuration saved as %s.\n") % (config_file, backup_file)
    else:
        message = message + _("Configuration was written to %s.\n") % (config_file)
    if moved_old_file:
        message = message + _("\nNote: The file /etc/X11/XF86Config-4 has been deprecated and we now always use /etc/X11/XF86Config. The XF86Config-4 file has been saved as XF86Config-4.deprecated.")
        
    m = gtk.MessageDialog(None, gtk.DIALOG_MODAL,
                          gtk.MESSAGE_INFO, gtk.BUTTONS_OK, message)
    m.set_position(gtk.WIN_POS_CENTER)
    m.run()
    m.destroy()

def fork_off_server(xconfig):
    tmp_config = tempfile.mktemp("xf86config")
    verbose(_("Writing temporary config file to %s") % tmp_config)
    xconfig.write(tmp_config)
    os.environ["DISPLAY"]=":17.0"
    verbose(_("Trying to start X server"))
    args = [ "/usr/X11R6/bin/XFree86", "-xf86config", tmp_config, "-ac", "-v", "-nolisten", "tcp", "-s", "1440", "-terminate",
             "-dpms", "-logfile", "/dev/null", ":17"]
    
    serverpid = os.fork()
    if serverpid == 0: #child
        try:
            err = os.open(logFile, os.O_RDWR | os.O_CREAT, 0644)
            if err < 0:
                sys.stderr.write(_("error opening %s\n" % logFile))
            else:
                os.dup2(err, 2)
                os.close(err)
        except:
            # oh well
            pass
        
        os.execv(args[0], args)
        sys.exit (1)
            
    return (serverpid, tmp_config)

def start_x_server(xconfig):
    try:
        (server, tmp_config) = fork_off_server(xconfig)
    except:
	import traceback
        server = None
	(type, value, tb) = sys.exc_info()
	list = traceback.format_exception (type, value, tb)
	text = string.joinfields (list, "")
	print text
    
    # give time for the server to fail (if it is going to fail...)
    if not server:
        verbose(_("X SERVER FAILED"));
        raise RuntimeError, "X server failed to start"

    count = 0

    if verbose_output:
        sys.stdout.write(_("Waiting for X server to start...log located in %s\n") % logFile)
        sys.stdout.flush()

    for i in range(5):
        time.sleep(1)
        if verbose_output:
            sys.stdout.write("%s..." % (i+1))
            sys.stdout.flush()
        
    while count < 60:
        if verbose_output:
            sys.stdout.write(".")
            sys.stdout.flush()
        pid = 0
        try:
            pid, status = os.waitpid (server, os.WNOHANG)
        except OSError, (errno, msg):
            print __name__, "waitpid:", msg
	if pid:
            verbose(_("X SERVER FAILED"));
	    raise RuntimeError, "X server failed to start"
	try:
	    os.stat ("/tmp/.X11-unix/X17")
	    break
	except OSError:
	    pass
	time.sleep(1)
	count = count + 1
  
    # We dont need the configuration file anymore.
    os.unlink(tmp_config)

    if count < 60:
        verbose (_("X server started successfully."))
        return server
    else:
        return 0

def handle_set_commands(set_values, hardware_state):
    # Set up values that limit resolution first
    if set_values.has_key("hsync"):
        hardware_state.set_hsync(set_values["hsync"])
    if set_values.has_key("vsync"):
        hardware_state.set_vsync(set_values["vsync"])
    if set_values.has_key("videoram"):
        hardware_state.set_videocard_ram(string.atoi(set_values["videoram"]))
    if set_values.has_key("driver"):
        hardware_state.set_videocard_driver(set_values["driver"])
    if set_values.has_key("card"):
	hardware_state.set_videocard_card(set_values["card"])
    if set_values.has_key("resolution"):
        hardware_state.set_resolution(set_values["resolution"])
    if set_values.has_key("depth"):
        depth = string.atoi(set_values["depth"])
        hardware_state.set_colordepth(depth)

def kill_x_server():
    if xserverpid:
        try:
            os.kill(xserverpid, 15)
            os.waitpid(xserverpid, 0)
        except:
            pass
   

def usage():
    print \
_("""Usage: redhat-config-xfree86 [OPTIONS]
Options:
  -h, --help        display this help and exit
  -v, --verbose     display what the program is doing more verbosely
  -o, --output=     the filename of the config file to be output
      --reconfig    don't base configuration on existing config files
      --noui        don't show the gui, implicit when using --set options
      --forceui     force the ui to be shown, needed when using --set options
                    to change some setting, but you still want to show the ui
      --set-<key>=  change the value of a specific configuration key.

        currently supported keys are:

        resolution  the screen resolution used
        depth       the color depth in bits
                    most drivers support 8, 15, 16 and 24
        driver      the graphics card driver to use
        card        the graphics card description to use
                    e.g. 'ATI Radeon (generic)'
        vsync       monitor vertical sync rates allowed (in Hz)
        hsync       monitor horizontal sync rates allowed (in kHz)
        videoram    the amount of videoram in kilobytes, 0 means probe
                    
""")

if __name__ == '__main__':
    signal.signal(signal.SIGINT, signal.SIG_DFL)

textdomain("redhat-config-xfree86")

output_file = "/etc/X11/XF86Config"
output_file_specified = FALSE

try:
    (opts, rest_args) = getopt.getopt(sys.argv[1:],
                               "vo:h",
                               ["help", "reconfig", "noui", "verbose",
                                "forceui", "output=",
                                "set-hsync=", "set-vsync=",
                                "set-videoram=", "set-driver=", "set-card=",
                                "set-resolution=", "set-depth="])
except (getopt.GetoptError), e:
    print e
    print
    usage()
    sys.exit(1)

reconfig = FALSE
no_ui = FALSE
force_ui = FALSE
set_values = {}
verbose_output = FALSE

for (opt, value) in opts:
    if opt == "-v" or opt == "--verbose":
        verbose_output = TRUE
    if opt == "--reconfig":
        reconfig = TRUE
    if opt == "-h" or opt == "--help":
        usage()
        sys.exit(1)
    if opt == "--noui":
        no_ui = TRUE
    if opt == "--forceui":
        force_ui = TRUE
    if opt == "-o" or opt == "--output":
        output_file = value
        output_file_specified = TRUE
    if opt[:6] == "--set-":
        no_ui = TRUE
        set_values[opt[6:]] = value

if force_ui:
    no_ui = FALSE

vc = rhpl.videocard.VideoCardInfo()
card_list = vc.videocards

card_list.insert(0, None)

for loop_count in range(len(card_list)):
    xconfig = None
    xconfigpath = None
    xserverpid = 0

    if reconfig and loop_count == 0:
        continue

    if not reconfig:
        (xconfig, xconfigpath) = xf86config.readConfigFile()
        if xconfig:
            verbose(_("Read configuration file %s") % xconfigpath)
        else:
            verbose(_("Could not find existing X configuration"))

    hardware_state = XF86HardwareState(xconfig)

    current_card =  card_list[loop_count]

    if current_card != None:
        #The first time thru the loop is not reconfig
        print _("Trying with card:"), current_card.getCardData()["NAME"]
        current_card_driver = current_card.getCardData()["DRIVER"]

        hardware_state.set_videocard_driver(current_card_driver)
        hardware_state.set_videocard_name(current_card.getCardData()["NAME"])

    if xconfig == None:
        xconfig = generate_xconfig(hardware_state)

    handle_set_commands(set_values, hardware_state)

    if not no_ui:
        no_x = FALSE
        try:
            import gtk
            reload(gtk)
        except:
            no_x = TRUE

        if no_x:
            # X not already running.  Try to bring up a server.
            try:
                xserverpid = start_x_server(xconfig)
            except:
                reconfig = TRUE
                xconfig = None
                if loop_count == len(card_list)-1:
                    print _("Error, cannot start X server.")
                    sys.exit(1)                    
                else:
                    print _("Couldn't start X server, trying with a fresh configuration")

                continue

        if xserverpid:
            # if we had to start an X server, make it look half-way decent.
            import gtk
            reload(gtk)        
            import gtk.gdk
            root = gtk.gdk.get_default_root_window()
            arrow = gtk.gdk.Cursor(gtk.gdk.LEFT_PTR)
            root.set_cursor (arrow)
            #Call xsri to set the background to something nice
            os.system("xsri --scale-width=100 --scale-height=100 /usr/share/gdm/themes/Bluecurve/lightrays.png")

        # Work around bug in pygtk where it sets LC_NUMERIC
        locale.setlocale(locale.LC_NUMERIC,"C")
        import gtk.glade
        # TODO: try to import gtk+, on fail, try to start new X
        import xConfigDialog
        dialog = xConfigDialog.XConfigDialog(hardware_state)
        dialog.update_ui()
        if not dialog.run():
            kill_x_server()
            sys.exit(1)
    break

hardware_state.merge_into(xconfig)

if xconfigpath and not output_file_specified:
    output_file = xconfigpath
    if output_file == "/etc/X11/XF86Config-4":
        output_file = "/etc/X11/XF86Config"

moved_old_file = FALSE
if os.access("/etc/X11/XF86Config-4", os.F_OK):
    verbose(_("Moving /etc/X11/XF86Config-4 to /etc/X11/XF86Config-4.deprecated"))
    os.rename("/etc/X11/XF86Config-4", "/etc/X11/XF86Config-4.deprecated")
    moved_old_file = TRUE
            
backup_file = None
if os.access(output_file, os.F_OK):
    backup_file = output_file + ".backup"
    verbose(_("Backing up %s to %s") % (output_file, backup_file))
    os.rename(output_file, backup_file)

verbose(_("Writing configuration to %s") % output_file)
xconfig.comment = "# XFree86 4 configuration created by redhat-config-xfree86\n"
xconfig.write(output_file)

if os.access("/etc/X11/X", os.F_OK):
    verbose(_("Removing old /etc/X11/X"))
    os.unlink("/etc/X11/X")

verbose(_("Creating /etc/X11/X symlink"))
os.symlink("../../usr/X11R6/bin/XFree86", "/etc/X11/X")


# Tell gdm to restart the server when the user logs out
# Otherwise the xserver won't re-read the config file
gdmfifofile = "/var/gdm/.gdmfifo"
if os.access(gdmfifofile, os.F_OK):
    verbose(_("Kicking gdm"))
    try:
        fd = os.open(gdmfifofile, os.O_WRONLY | os.O_APPEND | os.O_NONBLOCK)
        if fd >= 0:
            os.write(fd,"\nDIRTY_SERVERS\n");
            os.close(fd)
    except:
        pass

if not no_ui:
    finished_dialog(output_file, backup_file, moved_old_file)

kill_x_server()

